use std::collections::HashMap;
use std::iter::FromIterator;

fn create_orbit_map(input: &str) -> HashMap<String, String> {
    let mut orbit_map: HashMap<String, String> = HashMap::new();
    input
        .lines()
        .map(|line| line.split(")").map(String::from).collect())
        .for_each(|pair: Vec<String>| {
            orbit_map.entry(pair[1].clone()).or_insert(pair[0].clone());
        });
    orbit_map
}

fn part1(input: &str) -> usize {
    let orbit_map = create_orbit_map(input);

    let mut cnt: usize = 0;
    for obj in orbit_map.keys() {
        let mut key = obj;
        while let Some(val) = orbit_map.get(key) {
            cnt += 1;
            key = val;
        }
    }
    cnt
}

fn part2(input: &str) -> usize {
    let orbit_map = create_orbit_map(input);

    let my_traj: HashMap<String, usize> = HashMap::from_iter(
        std::iter::successors(orbit_map.get("YOU").map(|el| (el, 0)), |(nxt, n)| {
            (orbit_map.get(*nxt).map(|el| (el, n + 1 as usize)))
        })
        .map(|(el, n)| (String::from(el), n)),
    );

    let mut santa_transfers = 0;
    let mut santa = orbit_map.get("SAN").unwrap();
    loop {
        if my_traj.contains_key(santa) {
            return my_traj.get(santa).unwrap() + santa_transfers;
        } else {
            santa = orbit_map.get(santa).expect("no common orbits found");
            santa_transfers += 1;
        }
    }
}

pub fn main() -> std::io::Result<()> {
    let input = std::include_str!("../day6-input.txt").trim();
    println!("part1 = {:?}", part1(input));
    println!("part2 = {:?}", part2(input));
    Ok(())
}

#[test]
fn test_day6_part1() {
    let input = "COM)B\nB)C\nC)D\nD)E\nE)F\nB)G\nG)H\nD)I\nE)J\nJ)K\nK)L";
    assert_eq!(part1(input), 42);
}

#[test]
fn test_day6_part2() {
    let input = "COM)B\nB)C\nC)D\nD)E\nE)F\nB)G\nG)H\nD)I\nE)J\nJ)K\nK)L\nK)YOU\nI)SAN";
    assert_eq!(part2(input), 4);
}
