use crate::day5::compute_intcode_extended;

pub fn permutations(start: usize, size: usize) -> Permutations {
    Permutations {
        idxs: (start..size).collect(),
        swaps: vec![start; size],
        i: 0,
    }
}

pub struct Permutations {
    idxs: Vec<usize>,
    swaps: Vec<usize>,
    i: usize,
}

impl Iterator for Permutations {
    type Item = Vec<usize>;

    fn next(&mut self) -> Option<Self::Item> {
        if self.i > 0 {
            loop {
                if self.i >= self.swaps.len() {
                    return None;
                }
                if self.swaps[self.i] < self.i {
                    break;
                }
                self.swaps[self.i] = 0;
                self.i += 1;
            }
            self.idxs.swap(self.i, (self.i & 1) * self.swaps[self.i]);
            self.swaps[self.i] += 1;
        }
        self.i = 1;
        Some(self.idxs.clone())
    }
}

fn compute_thruster(input: &mut Vec<i32>, settings: Vec<usize>) -> i32 {
    let mut signal = 0;
    for phase in settings {
        signal = compute_intcode_extended(input, vec![phase as i32, signal]).1[0];
    }
    signal
}

fn part1() -> i32 {
    let input: Vec<i32> = std::include_str!("../day7-input.txt")
        .trim()
        .split(",")
        .map(|x| x.parse().ok().expect(&format!("{} not a number", x)))
        .collect();

    let mut max_thruster = std::i32::MIN;
    for settings in permutations(0, 5).collect::<Vec<_>>() {
        let thrust = compute_thruster(&mut input.clone(), settings);
        max_thruster = max_thruster.max(thrust);
    }
    max_thruster
}

fn compute_feedback(input: &mut Vec<i32>, mut settings: Vec<i32>) -> Vec<i32> {
    settings.push(0);
    println!("{:?}", settings);
    let signal = compute_intcode_extended(input, settings).1;
    println!("{:?}", signal);
    signal
}

fn part2() -> i32 {
    let mut input: Vec<i32> = std::include_str!("../day7-input.txt")
        .trim()
        .split(",")
        .map(|x| x.parse().ok().expect(&format!("{} not a number", x)))
        .collect();

    let mut max_thruster = std::i32::MIN;
    for settings in permutations(5, 5).collect::<Vec<_>>() {
        let thrust = compute_thruster(&mut input, settings);
        max_thruster = max_thruster.max(thrust);
    }
    max_thruster
}

pub fn main() {
    println!("part1 = {}", part1());
    println!("part2 = {}", part2());
}

#[test]
fn test_compute_thruster_1() {
    let mut input = vec![
        3, 15, 3, 16, 1002, 16, 10, 16, 1, 16, 15, 15, 4, 15, 99, 0, 0,
    ];
    let settings = vec![4, 3, 2, 1, 0];
    assert_eq!(compute_thruster(&mut input, settings), 43210);
}

#[test]
fn test_compute_thruster_2() {
    let mut input = vec![
        3, 23, 3, 24, 1002, 24, 10, 24, 1002, 23, -1, 23, 101, 5, 23, 23, 1, 24, 23, 23, 4, 23, 99,
        0, 0,
    ];
    let settings = vec![0, 1, 2, 3, 4];
    assert_eq!(compute_thruster(&mut input, settings), 54321);
}

#[test]
fn test_compute_thruster_3() {
    let mut input = vec![
        3, 31, 3, 32, 1002, 32, 10, 32, 1001, 31, -2, 31, 1007, 31, 0, 33, 1002, 33, 7, 33, 1, 33,
        31, 31, 1, 32, 31, 31, 4, 31, 99, 0, 0, 0,
    ];
    let settings = vec![1, 0, 4, 3, 2];
    assert_eq!(compute_thruster(&mut input, settings), 65210);
}

#[test]
fn test_compute_feedback_1() {
    let mut input = vec![
        3, 26, 1001, 26, -4, 26, 3, 27, 1002, 27, 2, 27, 1, 27, 26, 27, 4, 27, 1001, 28, -1, 28,
        1005, 28, 6, 99, 0, 0, 5,
    ];
    let settings = vec![5, 6, 7, 8, 9];
    assert_eq!(compute_feedback(&mut input, settings), [139629729]);
}

#[test]
fn test_compute_feedback_2() {
    let mut input = vec![
        3, 52, 1001, 52, -5, 52, 3, 53, 1, 52, 56, 54, 1007, 54, 5, 55, 1005, 55, 26, 1001, 54, -5,
        54, 1105, 1, 12, 1, 53, 54, 53, 1008, 54, 0, 55, 1001, 55, 1, 55, 2, 53, 55, 53, 4, 53,
        1001, 56, -1, 56, 1005, 56, 6, 99, 0, 0, 0, 0, 10,
    ];
    let settings = vec![9, 7, 8, 5, 6];
    assert_eq!(compute_feedback(&mut input, settings), [18216]);
}
